/* SPDX-License-Identifier: GPL-3.0-or-later */
/* Copyright 2025 Canonical Ltd */

#include <gio/gio.h>
#include <libnotify/notify.h>

G_DECLARE_FINAL_TYPE (NotificationAppLaunchContext,
                      notification_app_launch_context,
                      NOTIFICATION, APP_LAUNCH_CONTEXT, GAppLaunchContext);

GAppLaunchContext *notification_app_launch_context_new (NotifyNotification *notification);
