/* SPDX-License-Identifier: GPL-3.0-or-later */
/* Copyright 2025 Canonical Ltd */

#pragma once

#include "update-notifier.h"

void
unattended_upgrade_tray_icon_init (TrayApplet *tray_applet);

void
unattended_upgrade_tray_icon_dispose (TrayApplet *tray_applet);
