/* livepatch-utils.h
 * Copyright (C) 2019 Canonical Ltd
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor
 * Boston, MA  02110-1301 USA.
 */

#pragma once

#include <gio/gio.h>

#define LIVEPATCH_ERROR livepatch_error_quark()
#define LIVEPATCH_DESKTOP_FILE "software-properties-livepatch.desktop"

typedef enum
{
  LIVEPATCH_ERROR_CMD_FAILED,
  LIVEPATCH_ERROR_NOMATCH
} LivepatchError;


gboolean  livepatch_has_settings_ui ();
gboolean  livepatch_is_supported    ();
gboolean  livepatch_is_running      ();
gchar*    livepatch_get_state       (GError **error);
gchar*    livepatch_get_check_state (GError **error);
ssize_t   livepatch_get_num_fixes   (GError **error);
